@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
 
    Copyright (C) 1999 Reginald Stadlbauer <reggie@kde.org>
+   Copyright (C) 2017 Harald Sitter <sitter@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,49 +20,50 @@
 */
 
 #include "kcharselect.h"
-
 #include "kcharselect_p.h"
 
+#include "loggingcategory.h"
+
 #include <QAction>
 #include <QActionEvent>
+#include <QApplication>
+#include <QBoxLayout>
 #include <QComboBox>
 #include <QDebug>
 #include <QDoubleSpinBox>
+#include <QFontComboBox>
 #include <QHeaderView>
 #include <QLineEdit>
-#include <QBoxLayout>
-#include <QShortcut>
 #include <QSplitter>
-#include <QPushButton>
 #include <QToolButton>
 #include <QTextBrowser>
-#include <QFontComboBox>
+#include <QTimer>
 
 Q_GLOBAL_STATIC(KCharSelectData, s_data)
 
 class KCharSelectTablePrivate
 {
 public:
-    KCharSelectTablePrivate(KCharSelectTable *q): q(q), model(0)
+    KCharSelectTablePrivate(KCharSelectTable *q): q(q), model(nullptr)
     {}
 
     KCharSelectTable *q;
 
     QFont font;
     KCharSelectItemModel *model;
-    QVector<QChar> chars;
-    QChar chr;
+    QVector<uint> chars;
+    uint chr;
 
     void _k_resizeCells();
     void _k_doubleClicked(const QModelIndex &index);
     void _k_slotSelectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
 };
 
-class KCharSelect::KCharSelectPrivate
+class Q_DECL_HIDDEN KCharSelect::KCharSelectPrivate
 {
 public:
     struct HistoryItem {
-        QChar c;
+        uint c;
         bool fromSearch;
         QString searchString;
     };
@@ -70,11 +72,12 @@ public:
 
     KCharSelectPrivate(KCharSelect *q)
         : q(q)
-        , searchLine(0)
+        , searchLine(nullptr)
         , searchMode(false)
         , historyEnabled(false)
+        , allPlanesEnabled(false)
         , inHistory(0)
-        , actionParent(0)
+        , actionParent(nullptr)
     {
     }
 
@@ -85,32 +88,34 @@ public:
 
     KCharSelect *q;
 
-    QToolButton *backButton;
-    QToolButton *forwardButton;
-    QLineEdit *searchLine;
-    QFontComboBox *fontCombo;
-    QSpinBox *fontSizeSpinBox;
-    QComboBox *sectionCombo;
-    QComboBox *blockCombo;
-    KCharSelectTable *charTable;
-    QTextBrowser *detailBrowser;
+    QToolButton *backButton = nullptr;
+    QToolButton *forwardButton = nullptr;
+    QLineEdit *searchLine = nullptr;
+    QFontComboBox *fontCombo = nullptr;
+    QSpinBox *fontSizeSpinBox = nullptr;
+    QComboBox *sectionCombo = nullptr;
+    QComboBox *blockCombo = nullptr;
+    KCharSelectTable *charTable = nullptr;
+    QTextBrowser *detailBrowser = nullptr;
 
     bool searchMode; //a search is active
     bool historyEnabled;
+    bool allPlanesEnabled;
     int inHistory; //index of current char in history
     QList<HistoryItem> history;
     QObject *actionParent;
 
     QString createLinks(QString s);
-    void historyAdd(QChar c, bool fromSearch, const QString &searchString);
+    void historyAdd(uint c, bool fromSearch, const QString &searchString);
     void showFromHistory(int index);
     void updateBackForwardButtons();
     void _k_activateSearchLine();
     void _k_back();
     void _k_forward();
     void _k_fontSelected();
-    void _k_updateCurrentChar(QChar c);
-    void _k_slotUpdateUnicode(QChar c);
+    void _k_charSelected(uint c);
+    void _k_updateCurrentChar(uint c);
+    void _k_slotUpdateUnicode(uint c);
     void _k_sectionSelected(int index);
     void _k_blockSelected(int index);
     void _k_searchEditChanged();
@@ -128,7 +133,9 @@ KCharSelectTable::KCharSelectTable(QWidget *parent, const QFont &_font)
     d->font = _font;
 
     setTabKeyNavigation(false);
+    setSelectionBehavior(QAbstractItemView::SelectItems);
     setSelectionMode(QAbstractItemView::SingleSelection);
+
     QPalette _palette;
     _palette.setColor(backgroundRole(), palette().color(QPalette::Base));
     setPalette(_palette);
@@ -142,6 +149,7 @@ KCharSelectTable::KCharSelectTable(QWidget *parent, const QFont &_font)
     setAcceptDrops(true);
     setDropIndicatorShown(false);
     setDragDropMode(QAbstractItemView::DragDrop);
+    setTextElideMode(Qt::ElideNone);
 
     connect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(_k_doubleClicked(QModelIndex)));
 
@@ -163,7 +171,7 @@ void KCharSelectTable::setFont(const QFont &_font)
     d->_k_resizeCells();
 }
 
-QChar KCharSelectTable::chr()
+uint KCharSelectTable::chr()
 {
     return d->chr;
 }
@@ -173,12 +181,12 @@ QFont KCharSelectTable::font() const
     return d->font;
 }
 
-QVector<QChar> KCharSelectTable::displayedChars() const
+QVector<uint> KCharSelectTable::displayedChars() const
 {
     return d->chars;
 }
 
-void KCharSelectTable::setChar(QChar c)
+void KCharSelectTable::setChar(uint c)
 {
     int pos = d->chars.indexOf(c);
     if (pos != -1) {
@@ -186,21 +194,21 @@ void KCharSelectTable::setChar(QChar c)
     }
 }
 
-void KCharSelectTable::setContents(const QVector<QChar> &chars)
+void KCharSelectTable::setContents(const QVector<uint> &chars)
 {
     d->chars = chars;
 
-    KCharSelectItemModel *m = d->model;
+    auto oldModel = d->model;
     d->model = new KCharSelectItemModel(chars, d->font, this);
     setModel(d->model);
     d->_k_resizeCells();
-    QItemSelectionModel *selectionModel = new QItemSelectionModel(d->model);
-    setSelectionModel(selectionModel);
-    setSelectionBehavior(QAbstractItemView::SelectItems);
-    setSelectionMode(QAbstractItemView::SingleSelection);
-    connect(selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_k_slotSelectionChanged(QItemSelection,QItemSelection)));
-    connect(d->model, SIGNAL(showCharRequested(QChar)), this, SIGNAL(showCharRequested(QChar)));
-    delete m; // this should hopefully delete aold selection models too, since it is the parent of them (didn't track, if there are setParent calls somewhere. Check that (jowenn)
+
+    // Setting a model changes the selectionModel. Make sure to always reconnect.
+    connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+            this, SLOT(_k_slotSelectionChanged(QItemSelection,QItemSelection)));
+    connect(d->model, &KCharSelectItemModel::showCharRequested, this, &KCharSelectTable::showCharRequested);
+
+    delete oldModel; // The selection model is thrown away when the model gets destroyed().
 }
 
 void KCharSelectTable::scrollTo(const QModelIndex &index, ScrollHint hint)
@@ -220,10 +228,10 @@ void KCharSelectTablePrivate::_k_slotSelectionChanged(const QItemSelection &sele
         return;
     }
     QVariant temp = model->data(selected.indexes().at(0), KCharSelectItemModel::CharacterRole);
-    if (temp.type() != QVariant::Char) {
+    if (temp.type() != QVariant::UInt) {
         return;
     }
-    QChar c = temp.toChar();
+    uint c = temp.toUInt();
     chr = c;
     emit q->focusItemChanged(c);
 }
@@ -232,7 +240,21 @@ void KCharSelectTable::resizeEvent(QResizeEvent *e)
 {
     QTableView::resizeEvent(e);
     if (e->size().width() != e->oldSize().width()) {
-        d->_k_resizeCells();
+        // Resize our cells. But do so asynchronously through the event loop.
+        // Otherwise we can end up with an infinite loop as resizing the cells in turn results in
+        // a layout change which results in a resize event. More importantly doing this blockingly
+        // crashes QAccessible as the resize we potentially cause will discard objects which are
+        // still being used in the call chain leading to this event.
+        // https://bugs.kde.org/show_bug.cgi?id=374933
+        // https://bugreports.qt.io/browse/QTBUG-58153
+        // This can be removed once a fixed Qt version is the lowest requirement for Frameworks.
+        auto timer = new QTimer(this);
+        timer->setSingleShot(true);
+        connect(timer, &QTimer::timeout, [&,timer]() {
+            d->_k_resizeCells();
+            timer->deleteLater();
+        });
+        timer->start(0);
     }
 }
 
@@ -249,31 +271,31 @@ void KCharSelectTablePrivate::_k_resizeCells()
     // fontMetrics.maxWidth() doesn't help because of font fallbacks
     // (testcase: Malayalam characters)
     int maxCharWidth = 0;
-    const QVector<QChar> chars = model->chars();
-    for (int i = 0 ; i < chars.size(); ++i) {
-        QChar thisChar = chars.at(i);
+    const QVector<uint> chars = model->chars();
+    for (int i = 0; i < chars.size(); ++i) {
+        uint thisChar = chars.at(i);
         if(s_data()->isPrint(thisChar)) {
-            maxCharWidth = qMax(maxCharWidth, fontMetrics.boundingRect(QString(thisChar)).width());
+            maxCharWidth = qMax(maxCharWidth, fontMetrics.boundingRect(QString::fromUcs4(&thisChar, 1)).width());
         }
     }
     // Avoid too narrow cells
     maxCharWidth = qMax(maxCharWidth, 2 * fontMetrics.xHeight());
     maxCharWidth = qMax(maxCharWidth, fontMetrics.height());
     // Add the necessary padding, trying to match the delegate
-    const int textMargin = q->style()->pixelMetric(QStyle::PM_FocusFrameHMargin, 0, q) + 1;
+    const int textMargin = q->style()->pixelMetric(QStyle::PM_FocusFrameHMargin, nullptr, q) + 1;
     maxCharWidth += 2 * textMargin;
 
     const int columns = qMax(1, viewportWidth / maxCharWidth);
     model->setColumnCount(columns);
 
-    const QChar oldChar = q->chr();
+    const uint oldChar = q->chr();
 
     const int new_w = viewportWidth / columns;
     const int rows = model->rowCount();
     q->setUpdatesEnabled(false);
     QHeaderView *hHeader = q->horizontalHeader();
     const int spaceLeft = viewportWidth - new_w * columns;
-    for (int i = 0 ; i <= columns; i++ ) {
+    for (int i = 0; i <= columns; ++i) {
         if (i < spaceLeft) {
             hHeader->resizeSection(i, new_w + 1);
         } else {
@@ -291,7 +313,7 @@ void KCharSelectTablePrivate::_k_resizeCells()
     if (new_h < 5 || new_h < 4 + fontHeight) {
         new_h = qMax(5, 4 + fontHeight);
     }
-    for (int i = 0;i < rows;i++) {
+    for (int i = 0; i < rows; ++i) {
         vHeader->resizeSection(i, new_h);
     }
 
@@ -301,7 +323,7 @@ void KCharSelectTablePrivate::_k_resizeCells()
 
 void KCharSelectTablePrivate::_k_doubleClicked(const QModelIndex &index)
 {
-    QChar c = model->data(index, KCharSelectItemModel::CharacterRole).toChar();
+    uint c = model->data(index, KCharSelectItemModel::CharacterRole).toUInt();
     if (s_data()->isPrint(c)) {
         emit q->activated(c);
     }
@@ -309,24 +331,26 @@ void KCharSelectTablePrivate::_k_doubleClicked(const QModelIndex &index)
 
 void KCharSelectTable::keyPressEvent(QKeyEvent *e)
 {
-    if (d->model)
+    if (d->model) {
         switch (e->key()) {
         case Qt::Key_Space:
-            emit activated(QLatin1Char(' '));
+            emit activated(QChar::Space);
             return;
-            break;
-        case Qt::Key_Enter: case Qt::Key_Return: {
+        case Qt::Key_Enter:
+        case Qt::Key_Return: {
             if (!currentIndex().isValid()) {
                 return;
             }
-            QChar c = d->model->data(currentIndex(), KCharSelectItemModel::CharacterRole).toChar();
+            uint c = d->model->data(currentIndex(), KCharSelectItemModel::CharacterRole).toUInt();
             if (s_data()->isPrint(c)) {
                 emit activated(c);
             }
+            return;
         }
-        return;
-        break;
+        default:
+            break;
         }
+    }
     QTableView::keyPressEvent(e);
 }
 
@@ -338,7 +362,7 @@ void KCharSelectTable::keyPressEvent(QKeyEvent *e)
 KCharSelect::KCharSelect(QWidget *parent, const Controls controls)
     : QWidget(parent), d(new KCharSelectPrivate(this))
 {
-    initWidget(controls, NULL);
+    initWidget(controls, nullptr);
 }
 #endif
 
@@ -383,7 +407,7 @@ void KCharSelect::initWidget(const Controls controls, QObject *actionParent)
         d->searchLine->setToolTip(tr("Enter a search term or character here"));
 
         QAction *findAction = new QAction(this);
-        connect(findAction, SIGNAL(triggered(bool)), this, SLOT(_k_activateSearchLine()));
+        connect(findAction, &QAction::triggered, this, [this]() { d->_k_activateSearchLine(); });
         findAction->setObjectName(QStringLiteral("edit_find"));
         findAction->setText(tr("&Find..."));
         findAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-find")));
@@ -417,14 +441,14 @@ void KCharSelect::initWidget(const Controls controls, QObject *actionParent)
     d->forwardButton->setToolTip(tr("Next Character in History"));
 
     QAction *backAction = new QAction(this);
-    connect(backAction, SIGNAL(triggered(bool)), d->backButton, SLOT(animateClick()));
+    connect(backAction, &QAction::triggered, d->backButton, &QAbstractButton::animateClick);
     backAction->setObjectName(QStringLiteral("go_back"));
     backAction->setText(tr("&Back", "go back"));
     backAction->setIcon(QIcon::fromTheme(QStringLiteral("go-previous")));
     attachToActionParent(backAction, actionParent, QKeySequence::keyBindings(QKeySequence::Back));
 
     QAction *forwardAction = new QAction(this);
-    connect(forwardAction, SIGNAL(triggered(bool)), d->forwardButton, SLOT(animateClick()));
+    connect(forwardAction, &QAction::triggered, d->forwardButton, &QAbstractButton::animateClick);
     forwardAction->setObjectName(QStringLiteral("go_forward"));
     forwardAction->setText(tr("&Forward", "go forward"));
     forwardAction->setIcon(QIcon::fromTheme(QStringLiteral("go-next")));
@@ -436,8 +460,8 @@ void KCharSelect::initWidget(const Controls controls, QObject *actionParent)
         forwardAction->setIcon(tmp);
     }
 
-    connect(d->backButton, SIGNAL(clicked()), this, SLOT(_k_back()));
-    connect(d->forwardButton, SIGNAL(clicked()), this, SLOT(_k_forward()));
+    connect(d->backButton, &QToolButton::clicked, this, [this]() { d->_k_back(); });
+    connect(d->forwardButton, &QToolButton::clicked, this, [this]() { d->_k_forward(); });
 
     d->sectionCombo = new QComboBox(this);
     d->sectionCombo->setToolTip(tr("Select a category"));
@@ -508,12 +532,9 @@ void KCharSelect::initWidget(const Controls controls, QObject *actionParent)
 
     setCurrentFont(QFont());
 
-    connect(d->charTable, SIGNAL(focusItemChanged(QChar)), this, SLOT(_k_updateCurrentChar(QChar)));
-    connect(d->charTable, SIGNAL(activated(QChar)), this, SIGNAL(charSelected(QChar)));
-    connect(d->charTable, SIGNAL(focusItemChanged(QChar)),
-            this, SIGNAL(currentCharChanged(QChar)));
-
-    connect(d->charTable, SIGNAL(showCharRequested(QChar)), this, SLOT(setCurrentChar(QChar)));
+    connect(d->charTable, SIGNAL(focusItemChanged(uint)), this, SLOT(_k_updateCurrentChar(uint)));
+    connect(d->charTable, SIGNAL(activated(uint)), this, SLOT(_k_charSelected(uint)));
+    connect(d->charTable, SIGNAL(showCharRequested(uint)), this, SLOT(setCurrentCodePoint(uint)));
 
     d->detailBrowser = new QTextBrowser(this);
     if (DetailBrowser & controls) {
@@ -528,7 +549,7 @@ void KCharSelect::initWidget(const Controls controls, QObject *actionParent)
     setFocusProxy(d->charTable);
     d->_k_sectionSelected(0);
     d->_k_blockSelected(0);
-    setCurrentChar(0x0);
+    setCurrentCodePoint(QChar::Null);
 
     d->historyEnabled = true;
 }
@@ -550,7 +571,25 @@ void KCharSelect::setCurrentFont(const QFont &_font)
     d->_k_fontSelected();
 }
 
+void KCharSelect::setAllPlanesEnabled(bool all)
+{
+    d->allPlanesEnabled = all;
+}
+
+bool KCharSelect::allPlanesEnabled() const
+{
+    return d->allPlanesEnabled;
+}
+
 QChar KCharSelect::currentChar() const
+{
+    if (d->allPlanesEnabled) {
+        qFatal("You must use KCharSelect::currentCodePoint instead of KCharSelect::currentChar");
+    }
+    return QChar(d->charTable->chr());
+}
+
+uint KCharSelect::currentCodePoint() const
 {
     return d->charTable->chr();
 }
@@ -562,11 +601,39 @@ QFont KCharSelect::currentFont() const
 
 QList<QChar> KCharSelect::displayedChars() const
 {
-    return d->charTable->displayedChars().toList();
+    if (d->allPlanesEnabled) {
+        qFatal("You must use KCharSelect::displayedCodePoints instead of KCharSelect::displayedChars");
+    }
+    QList<QChar> result;
+    foreach (uint c, d->charTable->displayedChars()) {
+        result.append(QChar(c));
+    }
+    return result;
+}
+
+QVector<uint> KCharSelect::displayedCodePoints() const
+{
+    return d->charTable->displayedChars();
 }
 
 void KCharSelect::setCurrentChar(const QChar &c)
 {
+    if (d->allPlanesEnabled) {
+        qCritical("You should use KCharSelect::setCurrentCodePoint instead of KCharSelect::setCurrentChar");
+    }
+    setCurrentCodePoint(c.unicode());
+}
+
+void KCharSelect::setCurrentCodePoint(uint c)
+{
+    if (!d->allPlanesEnabled && QChar::requiresSurrogates(c)) {
+        qCritical("You must setAllPlanesEnabled(true) to use non-BMP characters");
+        c = QChar::ReplacementCharacter;
+    }
+    if (c > QChar::LastValidCodePoint) {
+        qCWarning(KWidgetsAddonsLog, "Code point outside Unicode range");
+        c = QChar::LastValidCodePoint;
+    }
     bool oldHistoryEnabled = d->historyEnabled;
     d->historyEnabled = false;
     int block = s_data()->blockIndex(c);
@@ -580,9 +647,9 @@ void KCharSelect::setCurrentChar(const QChar &c)
     d->charTable->setChar(c);
 }
 
-void KCharSelect::KCharSelectPrivate::historyAdd(QChar c, bool fromSearch, const QString &searchString)
+void KCharSelect::KCharSelectPrivate::historyAdd(uint c, bool fromSearch, const QString &searchString)
 {
-    //qDebug() << "about to add char" << c << "fromSearch" << fromSearch << "searchString" << searchString;
+    //qCDebug(KWidgetsAddonsLog) << "about to add char" << c << "fromSearch" << fromSearch << "searchString" << searchString;
 
     if (!historyEnabled) {
         return;
@@ -621,7 +688,7 @@ void KCharSelect::KCharSelectPrivate::showFromHistory(int index)
     updateBackForwardButtons();
 
     const HistoryItem &item = history[index];
-    //qDebug() << "index" << index << "char" << item.c << "fromSearch" << item.fromSearch
+    //qCDebug(KWidgetsAddonsLog) << "index" << index << "char" << item.c << "fromSearch" << item.fromSearch
     //    << "searchString" << item.searchString;
 
     //avoid adding an item from history into history again
@@ -635,7 +702,7 @@ void KCharSelect::KCharSelectPrivate::showFromHistory(int index)
         charTable->setChar(item.c);
     } else {
         searchLine->clear();
-        q->setCurrentChar(item.c);
+        q->setCurrentCodePoint(item.c);
     }
     historyEnabled = oldHistoryEnabled;
 }
@@ -672,8 +739,20 @@ void KCharSelect::KCharSelectPrivate::_k_fontSelected()
     emit q->currentFontChanged(font);
 }
 
-void KCharSelect::KCharSelectPrivate::_k_updateCurrentChar(QChar c)
+void KCharSelect::KCharSelectPrivate::_k_charSelected(uint c)
 {
+    if (!allPlanesEnabled) {
+        emit q->charSelected(QChar(c));
+    }
+    emit q->codePointSelected(c);
+}
+
+void KCharSelect::KCharSelectPrivate::_k_updateCurrentChar(uint c)
+{
+    if (!allPlanesEnabled) {
+        emit q->currentCharChanged(QChar(c));
+    }
+    emit q->currentCodePointChanged(c);
     if (searchMode) {
         //we are in search mode. make the two comboboxes show the section & block for this character.
         //(when we are not in search mode the current character always belongs to the current section & block.)
@@ -693,10 +772,10 @@ void KCharSelect::KCharSelectPrivate::_k_updateCurrentChar(QChar c)
     _k_slotUpdateUnicode(c);
 }
 
-void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(QChar c)
+void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(uint c)
 {
     QString html = QStringLiteral("<p>") + tr("Character:") + QLatin1Char(' ') + s_data()->display(c, charTable->font()) + QLatin1Char(' ') +
-                   s_data()->formatCode(c.unicode())  + QStringLiteral("<br />");
+                   s_data()->formatCode(c)  + QStringLiteral("<br />");
 
     QString name = s_data()->name(c);
     if (!name.isEmpty()) {
@@ -705,10 +784,11 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(QChar c)
     }
     QStringList aliases = s_data()->aliases(c);
     QStringList notes = s_data()->notes(c);
-    QVector<QChar> seeAlso = s_data()->seeAlso(c);
+    QVector<uint> seeAlso = s_data()->seeAlso(c);
     QStringList equivalents = s_data()->equivalents(c);
     QStringList approxEquivalents = s_data()->approximateEquivalents(c);
-    if (!(aliases.isEmpty() && notes.isEmpty() && seeAlso.isEmpty() && equivalents.isEmpty() && approxEquivalents.isEmpty())) {
+    QVector<uint> decomposition = s_data()->decomposition(c);
+    if (!(aliases.isEmpty() && notes.isEmpty() && seeAlso.isEmpty() && equivalents.isEmpty() && approxEquivalents.isEmpty() && decomposition.isEmpty())) {
         html += QStringLiteral("<p><b>") + tr("Annotations and Cross References") + QStringLiteral("</b></p>");
     }
 
@@ -730,12 +810,15 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(QChar c)
 
     if (!seeAlso.isEmpty()) {
         html += QStringLiteral("<p style=\"margin-bottom: 0px;\">") + tr("See also:") + QStringLiteral("</p><ul style=\"margin-top: 0px;\">");
-        foreach (const QChar &c2, seeAlso) {
-            html += QStringLiteral("<li><a href=\"") + QString::number(c2.unicode(), 16) + QStringLiteral("\">");
-            if (s_data()->isPrint(c2)) {
-                html += QStringLiteral("&#") + QString::number(c2.unicode()) + QStringLiteral("; ");
+        foreach (uint c2, seeAlso) {
+            if (!allPlanesEnabled && QChar::requiresSurrogates(c2)) {
+                continue;
             }
-            html += s_data()->formatCode(c2.unicode()) + QLatin1Char(' ') + s_data()->name(c2).toHtmlEscaped() + QStringLiteral("</a></li>");
+            html += QStringLiteral("<li><a href=\"") + QString::number(c2, 16) + QStringLiteral("\">");
+            if (s_data()->isPrint(c2)) {
+                html += QStringLiteral("&#8206;&#") + QString::number(c2) + QStringLiteral("; ");
+            }
+            html += s_data()->formatCode(c2) + QLatin1Char(' ') + s_data()->name(c2).toHtmlEscaped() + QStringLiteral("</a></li>");
         }
         html += QStringLiteral("</ul>");
     }
@@ -756,6 +839,17 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(QChar c)
         html += QStringLiteral("</ul>");
     }
 
+    if (!decomposition.isEmpty()) {
+        html += QStringLiteral("<p style=\"margin-bottom: 0px;\">") +  tr("Decomposition:") + QStringLiteral("</p><ul style=\"margin-top: 0px;\">");
+        foreach (uint c2, decomposition) {
+            if (!allPlanesEnabled && QChar::requiresSurrogates(c2)) {
+                continue;
+            }
+            html += QStringLiteral("<li>") + createLinks(s_data()->formatCode(c2, 4, QStringLiteral(""))) + QStringLiteral("</li>");
+        }
+        html += QStringLiteral("</ul>");
+    }
+
     QStringList unihan = s_data()->unihanInfo(c);
     if (unihan.count() == 7) {
         html += QStringLiteral("<p><b>") + tr("CJK Ideograph Information") + QStringLiteral("</b></p><p>");
@@ -813,26 +907,32 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(QChar c)
     html += tr("Block: ") + s_data()->block(c) + QStringLiteral("<br>");
     html += tr("Unicode category: ") + s_data()->categoryText(s_data()->category(c)) + QStringLiteral("</p>");
 
-    QByteArray utf8 = QString(c).toUtf8();
+    QByteArray utf8 = QString::fromUcs4(&c, 1).toUtf8();
 
     html += QStringLiteral("<p><b>") + tr("Various Useful Representations") + QStringLiteral("</b><br>");
     html += tr("UTF-8:");
     foreach (unsigned char c, utf8) {
         html += QLatin1Char(' ') + s_data()->formatCode(c, 2, QStringLiteral("0x"));
     }
-    html += QStringLiteral("<br>") + tr("UTF-16: ") + s_data()->formatCode(c.unicode(), 4, QStringLiteral("0x")) + QStringLiteral("<br>");
-    html += tr("C octal escaped UTF-8: ");
+    html += QStringLiteral("<br>") + tr("UTF-16: ");
+    if (QChar::requiresSurrogates(c)) {
+        html += s_data()->formatCode(QChar::highSurrogate(c), 4, QStringLiteral("0x"));
+        html += QLatin1Char(' ') + s_data->formatCode(QChar::lowSurrogate(c), 4, QStringLiteral("0x"));
+    } else {
+        html += s_data()->formatCode(c, 4, QStringLiteral("0x"));
+    }
+    html += QStringLiteral("<br>") + tr("C octal escaped UTF-8: ");
     foreach (unsigned char c, utf8) {
         html += s_data()->formatCode(c, 3, QStringLiteral("\\"), 8);
     }
-    html += QStringLiteral("<br>") + tr("XML decimal entity:") + QStringLiteral(" &amp;#") + QString::number(c.unicode()) + QStringLiteral(";</p>");
+    html += QStringLiteral("<br>") + tr("XML decimal entity:") + QStringLiteral(" &amp;#") + QString::number(c) + QStringLiteral(";</p>");
 
     detailBrowser->setHtml(html);
 }
 
 QString KCharSelect::KCharSelectPrivate::createLinks(QString s)
 {
-    QRegExp rx(QStringLiteral("\\b([\\dABCDEF]{4})\\b"));
+    QRegExp rx(QStringLiteral("\\b([\\dABCDEF]{4,5})\\b"));
 
     QStringList chars;
     int pos = 0;
@@ -844,13 +944,16 @@ QString KCharSelect::KCharSelectPrivate::createLinks(QString s)
 
     QSet<QString> chars2 = QSet<QString>::fromList(chars);
     foreach (const QString &c, chars2) {
-        int unicode = c.toInt(0, 16);
+        int unicode = c.toInt(nullptr, 16);
+        if (!allPlanesEnabled && QChar::requiresSurrogates(unicode)) {
+            continue;
+        }
         QString link = QStringLiteral("<a href=\"") + c + QStringLiteral("\">");
-        if (s_data()->isPrint(QChar(unicode))) {
-            link += QStringLiteral("&#") + QString::number(unicode) + QStringLiteral(";&nbsp;");
+        if (s_data()->isPrint(unicode)) {
+            link += QStringLiteral("&#8206;&#") + QString::number(unicode) + QStringLiteral(";&nbsp;");
         }
         link += QStringLiteral("U+") + c + QLatin1Char(' ');
-        link += s_data()->name(QChar(unicode)).toHtmlEscaped() + QStringLiteral("</a>");
+        link += s_data()->name(unicode).toHtmlEscaped() + QStringLiteral("</a>");
         s.replace(c, link);
     }
     return s;
@@ -861,6 +964,12 @@ void KCharSelect::KCharSelectPrivate::_k_sectionSelected(int index)
     blockCombo->clear();
     QVector<int> blocks = s_data()->sectionContents(index);
     foreach (int block, blocks) {
+        if (!allPlanesEnabled) {
+            const QVector<uint> contents = s_data()->blockContents(block);
+            if (!contents.isEmpty() && QChar::requiresSurrogates(contents.at(0))) {
+                continue;
+            }
+        }
         blockCombo->addItem(s_data()->blockName(block), QVariant(block));
     }
     blockCombo->setCurrentIndex(0);
@@ -878,10 +987,7 @@ void KCharSelect::KCharSelectPrivate::_k_blockSelected(int index)
     }
 
     int block = blockCombo->itemData(index).toInt();
-    const QVector<QChar> contents = s_data()->blockContents(block);
-    if (contents.count() <= index) {
-        return;
-    }
+    const QVector<uint> contents = s_data()->blockContents(block);
     charTable->setContents(contents);
     emit q->displayedCharsChanged();
     charTable->setChar(contents[0]);
@@ -895,12 +1001,12 @@ void KCharSelect::KCharSelectPrivate::_k_searchEditChanged()
 
         //upon leaving search mode, keep the same character selected
         searchMode = false;
-        QChar c = charTable->chr();
+        uint c = charTable->chr();
         bool oldHistoryEnabled = historyEnabled;
         historyEnabled = false;
         _k_blockSelected(blockCombo->currentIndex());
         historyEnabled = oldHistoryEnabled;
-        q->setCurrentChar(c);
+        q->setCurrentCodePoint(c);
     } else {
         sectionCombo->setEnabled(false);
         blockCombo->setEnabled(false);
@@ -918,7 +1024,17 @@ void KCharSelect::KCharSelectPrivate::_k_search()
         return;
     }
     searchMode = true;
-    const QVector<QChar> contents = s_data()->find(searchLine->text());
+    QVector<uint> contents = s_data()->find(searchLine->text());
+    if (!allPlanesEnabled) {
+        QVector<uint>::iterator it = contents.begin();
+        while (it != contents.end()) {
+            if (QChar::requiresSurrogates(*it)) {
+                it = contents.erase(it);
+            } else {
+                ++it;
+            }
+        }
+    }
     charTable->setContents(contents);
     emit q->displayedCharsChanged();
     if (!contents.isEmpty()) {
@@ -929,12 +1045,15 @@ void KCharSelect::KCharSelectPrivate::_k_search()
 void  KCharSelect::KCharSelectPrivate::_k_linkClicked(QUrl url)
 {
     QString hex = url.toString();
-    if (hex.size() > 4) {
+    if (hex.size() > 6) {
+        return;
+    }
+    int unicode = hex.toInt(nullptr, 16);
+    if (unicode > QChar::LastValidCodePoint) {
         return;
     }
-    int unicode = hex.toInt(0, 16);
     searchLine->clear();
-    q->setCurrentChar(QChar(unicode));
+    q->setCurrentCodePoint(unicode);
 }
 
 ////
@@ -950,22 +1069,22 @@ QVariant KCharSelectItemModel::data(const QModelIndex &index, int role) const
         return QVariant();
     }
 
-    QChar c = m_chars[pos];
+    uint c = m_chars[pos];
     if (role == Qt::ToolTipRole) {
         QString result = s_data()->display(c, m_font) + QStringLiteral("<br />") + s_data()->name(c).toHtmlEscaped() + QStringLiteral("<br />") +
-                         tr("Unicode code point:") + QLatin1Char(' ') + s_data()->formatCode(c.unicode()) + QStringLiteral("<br />") +
-                         tr("In decimal", "Character") + QLatin1Char(' ') + QString::number(c.unicode());
+                         tr("Unicode code point:") + QLatin1Char(' ') + s_data()->formatCode(c) + QStringLiteral("<br />") +
+                         tr("In decimal", "Character") + QLatin1Char(' ') + QString::number(c);
         return QVariant(result);
     } else if (role == Qt::TextAlignmentRole) {
         return QVariant(Qt::AlignHCenter | Qt::AlignVCenter);
     } else if (role == Qt::DisplayRole) {
         if (s_data()->isPrint(c)) {
-            return QVariant(c);
+            return QVariant(QString::fromUcs4(&c, 1));
         }
         return QVariant();
     } else if (role == Qt::BackgroundColorRole) {
         QFontMetrics fm = QFontMetrics(m_font);
-        if (fm.inFont(c) && s_data()->isPrint(c)) {
+        if (fm.inFontUcs4(c) && s_data()->isPrint(c)) {
             return QVariant(qApp->palette().color(QPalette::Base));
         } else {
             return QVariant(qApp->palette().color(QPalette::Button));
@@ -997,12 +1116,15 @@ bool KCharSelectItemModel::dropMimeData(const QMimeData *data, Qt::DropAction ac
     if (text.isEmpty()) {
         return false;
     }
-    emit showCharRequested(text[0]);
+    emit showCharRequested(text.toUcs4().at(0));
     return true;
 }
 
 void KCharSelectItemModel::setColumnCount(int columns)
 {
+    if (columns == m_columns) {
+        return;
+    }
     emit layoutAboutToBeChanged();
     m_columns = columns;
     emit layoutChanged();
