@@ -83,7 +83,7 @@ KDatePickerPrivateYearSelector::KDatePickerPrivateYearSelector(
     //               calendar->year( calendar->latestValidDate() ) );
     setValidator(val);
 
-    connect(this, SIGNAL(returnPressed()), SLOT(yearEnteredSlot()));
+    connect(this, &QLineEdit::returnPressed, this, &KDatePickerPrivateYearSelector::yearEnteredSlot);
 }
 
 void KDatePickerPrivateYearSelector::yearEnteredSlot()
@@ -118,11 +118,11 @@ void KDatePickerPrivateYearSelector::setYear(int year)
     setText(QString::number(year));
 }
 
-class KDatePicker::KDatePickerPrivate
+class Q_DECL_HIDDEN KDatePicker::KDatePickerPrivate
 {
 public:
     KDatePickerPrivate(KDatePicker *q) :
-        q(q), closeButton(0L), selectWeek(0L), todayButton(0), navigationLayout(0)
+        q(q), closeButton(nullptr), selectWeek(nullptr), todayButton(nullptr), navigationLayout(nullptr)
     {
     }
 
@@ -289,7 +289,7 @@ void KDatePicker::initWidget(const QDate &date_)
     d->selectWeek = new QComboBox(this);    // read only week selection
     d->selectWeek->setFocusPolicy(Qt::NoFocus);
     d->todayButton = new QToolButton(this);
-    d->todayButton->setIcon(QIcon::fromTheme(QLatin1String("go-jump-today")));
+    d->todayButton->setIcon(QIcon::fromTheme(QStringLiteral("go-jump-today")));
 
     d->yearForward->setToolTip(tr("Next year"));
     d->yearBackward->setToolTip(tr("Previous year"));
@@ -305,28 +305,28 @@ void KDatePicker::initWidget(const QDate &date_)
     d->line->setValidator(d->val);
     d->line->installEventFilter(this);
     if (QApplication::isRightToLeft()) {
-        d->yearForward->setIcon(QIcon::fromTheme(QLatin1String("arrow-left-double")));
-        d->yearBackward->setIcon(QIcon::fromTheme(QLatin1String("arrow-right-double")));
-        d->monthForward->setIcon(QIcon::fromTheme(QLatin1String("arrow-left")));
-        d->monthBackward->setIcon(QIcon::fromTheme(QLatin1String("arrow-right")));
+        d->yearForward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left-double")));
+        d->yearBackward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right-double")));
+        d->monthForward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
+        d->monthBackward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
     } else {
-        d->yearForward->setIcon(QIcon::fromTheme(QLatin1String("arrow-right-double")));
-        d->yearBackward->setIcon(QIcon::fromTheme(QLatin1String("arrow-left-double")));
-        d->monthForward->setIcon(QIcon::fromTheme(QLatin1String("arrow-right")));
-        d->monthBackward->setIcon(QIcon::fromTheme(QLatin1String("arrow-left")));
+        d->yearForward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right-double")));
+        d->yearBackward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left-double")));
+        d->monthForward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
+        d->monthBackward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
     }
 
     connect(d->table, SIGNAL(dateChanged(QDate)), SLOT(dateChangedSlot(QDate)));
-    connect(d->table, SIGNAL(tableClicked()), SLOT(tableClickedSlot()));
-    connect(d->monthForward, SIGNAL(clicked()), SLOT(monthForwardClicked()));
-    connect(d->monthBackward, SIGNAL(clicked()), SLOT(monthBackwardClicked()));
-    connect(d->yearForward, SIGNAL(clicked()), SLOT(yearForwardClicked()));
-    connect(d->yearBackward, SIGNAL(clicked()), SLOT(yearBackwardClicked()));
+    connect(d->table, &KDateTable::tableClicked, this, &KDatePicker::tableClickedSlot);
+    connect(d->monthForward, &QAbstractButton::clicked, this, &KDatePicker::monthForwardClicked);
+    connect(d->monthBackward, &QAbstractButton::clicked, this, &KDatePicker::monthBackwardClicked);
+    connect(d->yearForward, &QAbstractButton::clicked, this, &KDatePicker::yearForwardClicked);
+    connect(d->yearBackward, &QAbstractButton::clicked, this, &KDatePicker::yearBackwardClicked);
     connect(d->selectWeek, SIGNAL(activated(int)), SLOT(weekSelected(int)));
-    connect(d->todayButton, SIGNAL(clicked()), SLOT(todayButtonClicked()));
-    connect(d->selectMonth, SIGNAL(clicked()), SLOT(selectMonthClicked()));
-    connect(d->selectYear, SIGNAL(toggled(bool)), SLOT(selectYearClicked()));
-    connect(d->line, SIGNAL(returnPressed()), SLOT(lineEnterPressed()));
+    connect(d->todayButton, &QAbstractButton::clicked, this, &KDatePicker::todayButtonClicked);
+    connect(d->selectMonth, &QAbstractButton::clicked, this, &KDatePicker::selectMonthClicked);
+    connect(d->selectYear, &QAbstractButton::toggled, this, &KDatePicker::selectYearClicked);
+    connect(d->line, &QLineEdit::returnPressed, this, &KDatePicker::lineEnterPressed);
 
     topLayout->addWidget(d->table);
 
@@ -468,7 +468,7 @@ void KDatePicker::selectMonthClicked()
     }
 
     // cancelled
-    if ((item = popup.exec(d->selectMonth->mapToGlobal(QPoint(0, 0)), item)) == 0) {
+    if ((item = popup.exec(d->selectMonth->mapToGlobal(QPoint(0, 0)), item)) == nullptr) {
         return;
     }
 
@@ -629,7 +629,7 @@ int KDatePicker::fontSize() const
 
 void KDatePicker::setCloseButton(bool enable)
 {
-    if (enable == (d->closeButton != 0L)) {
+    if (enable == (d->closeButton != nullptr)) {
         return;
     }
 
@@ -640,12 +640,12 @@ void KDatePicker::setCloseButton(bool enable)
         d->navigationLayout->addSpacing(spacingHint);
         d->navigationLayout->addWidget(d->closeButton);
         d->closeButton->setToolTip(tr("Close", "@action:button"));
-        d->closeButton->setIcon(QIcon::fromTheme(QLatin1String("window-close")));
-        connect(d->closeButton, SIGNAL(clicked()),
-                topLevelWidget(), SLOT(close()));
+        d->closeButton->setIcon(QIcon::fromTheme(QStringLiteral("window-close")));
+        connect(d->closeButton, &QAbstractButton::clicked,
+                topLevelWidget(), &QWidget::close);
     } else {
         delete d->closeButton;
-        d->closeButton = 0L;
+        d->closeButton = nullptr;
     }
 
     updateGeometry();
