@@ -36,26 +36,28 @@ class KEditListWidgetPrivate
 {
 public:
     KEditListWidgetPrivate(KEditListWidget *parent)
-        : lineEdit(0),
-          editingWidget(0),
+        : lineEdit(nullptr),
+          editingWidget(nullptr),
           q(parent)
     {
     }
-    QListView *listView;
-    QPushButton *servUpButton, *servDownButton;
-    QPushButton *servNewButton, *servRemoveButton;
-    QLineEdit *lineEdit;
-    QWidget *editingWidget;
-    QVBoxLayout *mainLayout;
-    QVBoxLayout *btnsLayout;
-    QStringListModel *model;
+    QListView *listView = nullptr;
+    QPushButton *servUpButton = nullptr;
+    QPushButton *servDownButton = nullptr;
+    QPushButton *servNewButton = nullptr;
+    QPushButton *servRemoveButton = nullptr;
+    QLineEdit *lineEdit = nullptr;
+    QWidget *editingWidget = nullptr;
+    QVBoxLayout *mainLayout = nullptr;
+    QVBoxLayout *btnsLayout = nullptr;
+    QStringListModel *model = nullptr;
 
     bool checkAtEntering;
     KEditListWidget::Buttons buttons;
 
     void init(bool check = false, KEditListWidget::Buttons buttons = KEditListWidget::All,
-              QWidget *representationWidget = 0);
-    void setEditor(QLineEdit *lineEdit, QWidget *representationWidget = 0);
+              QWidget *representationWidget = nullptr);
+    void setEditor(QLineEdit *lineEdit, QWidget *representationWidget = nullptr);
     void updateButtonState();
     QModelIndex selectedIndex();
 
@@ -68,7 +70,7 @@ void KEditListWidgetPrivate::init(bool check, KEditListWidget::Buttons newButton
 {
     checkAtEntering = check;
 
-    servNewButton = servRemoveButton = servUpButton = servDownButton = 0L;
+    servNewButton = servRemoveButton = servUpButton = servDownButton = nullptr;
     q->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
                                  QSizePolicy::Preferred));
 
@@ -90,7 +92,7 @@ void KEditListWidgetPrivate::init(bool check, KEditListWidget::Buttons newButton
 
     setEditor(lineEdit, representationWidget);
 
-    buttons = 0;
+    buttons = KEditListWidget::Buttons();
     q->setButtons(newButtons);
 
     q->connect(listView->selectionModel(), &QItemSelectionModel::selectionChanged,
@@ -147,15 +149,19 @@ void KEditListWidgetPrivate::setEditor(QLineEdit *newLineEdit, QWidget *represen
 
 void KEditListWidgetPrivate::updateButtonState()
 {
-    QModelIndex index = selectedIndex();
-    if (servUpButton) {
-        servUpButton->setEnabled(index.isValid());
-    }
-    if (servDownButton) {
-        servDownButton->setEnabled(index.isValid());
-    }
-    if (servRemoveButton) {
-        servRemoveButton->setEnabled(index.isValid());
+    const bool hasSelectedItem = selectedIndex().isValid();
+
+    // TODO: merge with enableMoveButtons()
+    QPushButton * const buttons[3] = { servUpButton, servDownButton, servRemoveButton };
+
+    for (QPushButton *button : buttons) {
+        if (button) {
+            // keep focus in widget
+            if (!hasSelectedItem && button->hasFocus()) {
+                lineEdit->setFocus(Qt::OtherFocusReason);
+            }
+            button->setEnabled(hasSelectedItem);
+        }
     }
 }
 
@@ -170,13 +176,13 @@ QModelIndex KEditListWidgetPrivate::selectedIndex()
     }
 }
 
-class KEditListWidget::CustomEditorPrivate
+class Q_DECL_HIDDEN KEditListWidget::CustomEditorPrivate
 {
 public:
     CustomEditorPrivate(KEditListWidget::CustomEditor *q):
         q(q),
-        representationWidget(0),
-        lineEdit(0) {}
+        representationWidget(nullptr),
+        lineEdit(nullptr) {}
 
     KEditListWidget::CustomEditor *q;
     QWidget *representationWidget;
@@ -304,7 +310,7 @@ void KEditListWidget::setButtons(Buttons buttons)
         d->btnsLayout->insertWidget(0, d->servNewButton);
     } else if ((buttons & Add) == 0 && d->servNewButton) {
         delete d->servNewButton;
-        d->servNewButton = 0;
+        d->servNewButton = nullptr;
     }
 
     if ((buttons & Remove) && !d->servRemoveButton) {
@@ -316,7 +322,7 @@ void KEditListWidget::setButtons(Buttons buttons)
         d->btnsLayout->insertWidget(1, d->servRemoveButton);
     } else if ((buttons & Remove) == 0 && d->servRemoveButton) {
         delete d->servRemoveButton;
-        d->servRemoveButton = 0;
+        d->servRemoveButton = nullptr;
     }
 
     if ((buttons & UpDown) && !d->servUpButton) {
@@ -333,8 +339,8 @@ void KEditListWidget::setButtons(Buttons buttons)
         d->btnsLayout->insertWidget(2, d->servUpButton);
         d->btnsLayout->insertWidget(3, d->servDownButton);
     } else if ((buttons & UpDown) == 0 && d->servUpButton) {
-        delete d->servUpButton; d->servUpButton = 0;
-        delete d->servDownButton; d->servDownButton = 0;
+        delete d->servUpButton; d->servUpButton = nullptr;
+        delete d->servDownButton; d->servDownButton = nullptr;
     }
 
     d->buttons = buttons;
@@ -469,6 +475,14 @@ void KEditListWidget::addItem()
         }
     }
     if (d->servNewButton) {
+        // prevent losing the focus by it being moved outside of this widget
+        // as well as support the user workflow a little by moving the focus
+        // to the lineedit. chances are that users will add some items consecutively,
+        // so this will save a manual focus change, and it is also consistent
+        // to what happens on the click on the Remove button
+        if (d->servNewButton->hasFocus()) {
+            d->lineEdit->setFocus(Qt::OtherFocusReason);
+        }
         d->servNewButton->setEnabled(false);
     }
 
@@ -515,6 +529,14 @@ void KEditListWidget::removeItem()
     }
 
     if (currentIndex.row() >= 0) {
+        // prevent losing the focus by it being moved outside of this widget
+        // as well as support the user workflow a little by moving the focus
+        // to the lineedit. chances are that users will add some item next,
+        // so this will save a manual focus change,
+        if (d->servRemoveButton && d->servRemoveButton->hasFocus()) {
+            d->lineEdit->setFocus(Qt::OtherFocusReason);
+        }
+
         QString removedText = d->model->data(currentIndex, Qt::DisplayRole).toString();
 
         d->model->removeRows(currentIndex.row(), 1);
